import ply.lex as lex
import ply.yacc as yacc

tokens = ('WORD', 'STRING', 'COMMENT', 'KEYWORD', 'HERE', 'USER', 'IS', 
        'NOISE')

keywords = ('poll','proto','auth','via')
noise = ('and','with','wants','has')

t_COMMENT =r'^\#'
t_STRING = r'\".*?\"'
t_ignore = ' \t'

def t_WORD(t):
    r'[A-Za-z]+'
    if t.value in keywords:
        t.type = 'KEYWORD'
    elif t.value in noise:
        t.type = 'NOISE'
    elif t.value == 'user':
        t.type = 'USER'
    elif t.value == 'is':
        t.type = 'IS'
    elif t.value in ('here', 'there'):
        t.type = 'HERE'
    else:
        t.type = 'STRING'
    return t


def t_error(t):
   print 'Lex error %s' % t


def p_statement(p):
   "statement : expression"
   p[0] = p[1]

def p_statement_user(p):
    """statement : user
                 | user expr
                 | user NOISE expr"""
    if len(p) == 2:
        p[0] = p[1]
    elif len(p) == 3:
        p[0] = p[1]
        p[0].update(p[2])
    else:
        p[0] = p[1]
        p[0].update(p[3])

def p_statement_expr(p):
    "statement : expr"
    p[0] = p[1]

def p_comment(p):
   """expression : COMMENT phrase
                 | COMMENT STRING
                 | COMMENT WORD"""
   p[0] = { '#' : p[2] }

def p_phrase(p):
   """phrase : WORD WORD
             | WORD STRING
             | STRING WORD
             | WORD phrase
             | STRING STRING
             | STRING phrase"""
   p[0] = p[1] + " " + p[2]

def p_user(p):
    """user : USER STRING IS STRING
            | USER STRING HERE IS STRING
            | USER STRING IS STRING HERE
            | USER STRING HERE IS STRING HERE"""
    if len(p) == 5:
        p[0] = { 'user' : p[2], 'is' : p[4] }
    elif len(p) == 7:
        p[0] = { 'user' : [p[2], p[3]], 'is' : [p[5], p[6]] }
    elif p[3] == 'is':
        p[0] = { 'user' : p[2], 'is' : [p[4], p[5]] }
    else:
        p[0] = { 'user' : [p[2], p[3]], 'is' : p[4] }

def p_expr(p):
    """expr : KEYWORD STRING
          | expr NOISE expr
          | expr expr"""
    if type(p[1]) is dict:
        p[0] = p[1]
        if len(p) == 3:
            p[0].update(p[2])
        elif len(p) == 4:
            p[0].update(p[3])
    else:
        p[0] = { p[1] : p[2] }

def p_error(p):
    print 'Parse error in %s' % p


Lexer = lex.lex()
Parser = yacc.yacc()

test =  [ '# comment', '# "comment string"', '# comment string',
        'poll "host" via "host2" with proto "IMAP"',
        'user James here is someone there']
for t in test:
    print Parser.parse(t)
